﻿Imports System.Runtime.InteropServices

Module Module_ScrollHandler

    ' ------------------------------------------------------------------------------------------------------------
    ' The SCROLLINFO structure contains scroll bar parameters to be set by SetScrollInfo. 
    ' ------------------------------------------------------------------------------------------------------------
    ' Member descriptions:
    ' ------------------------------------------------------------------------------------------------------------
    ' cbSize: The number of bytes in the SCROLLINFO structure. This value must be filled in when setting and querying parameters.
    ' fMask: Specifies the members in the structure that are valid. The value has five options. You can combine multiple options with "or". You must set and query parameters.
    '   SIF_ALL: The entire structure is valid.
    '   SIF_DISABLENOSCROLL: This value is only used when the parameter is set. You can select members of the structure based on the control parameter settings.
    '   SIF_PAGE: The nPage member is valid.
    '   SIF_POS: Valid for nPos members.
    '   SIF_RANGE: Valid for nMin and nMax members.
    ' nMin: Minimum scrolling range.
    ' nMax: Maximum scroll range.
    ' nPage: Page size, used to determine the size of the proportional scroll box.
    ' nPos: Position of the scroll box.
    ' nTrackPos: The position of the scroll box when dragging. This parameter can only be queried and cannot be set.

    ' ============================================================================================================
    '  GET SCROLL POS
    ' ============================================================================================================
    Private Declare Function GetScrollInfo _
                             Lib "user32" Alias "GetScrollInfo" _
                             (ByVal hWnd As IntPtr, _
                              ByVal n As Integer, _
                              <MarshalAs(UnmanagedType.Struct)> ByRef lpScrollInfo As SCROLLINFO) As Integer

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure SCROLLINFO
        Public cbSize As Integer
        Public fMask As Integer
        Public nMin As Integer
        Public nMax As Integer
        Public nPage As Integer
        Public nPos As Integer
        Public nTrackPos As Integer
    End Structure

    Private Const SB_ENDSCROLL As Integer = 8
    Private Const SBS_HORZ As Integer = 0
    Private Const SBS_VERT As Integer = 1
    Private Const SIF_RANGE As Integer = 1
    Private Const SIF_PAGE As Integer = 2
    Private Const SIF_POS As Integer = 4
    Private Const SIF_TRACKPOS As Integer = 10
    Private Const SIF_ALL As Integer = (SIF_RANGE Or SIF_PAGE Or SIF_POS Or SIF_TRACKPOS)

    Friend Function GetScrollPosX(ByVal hWnd As IntPtr) As Int32
        Dim si As New SCROLLINFO
        si.fMask = SIF_ALL
        si.cbSize = Marshal.SizeOf(si)
        GetScrollInfo(hWnd, SBS_HORZ, si)
        Return si.nPos
    End Function

    Friend Function GetScrollPosY(ByVal hWnd As IntPtr) As Int32
        Dim si As New SCROLLINFO
        si.fMask = SIF_ALL
        si.cbSize = Marshal.SizeOf(si)
        GetScrollInfo(hWnd, SBS_VERT, si)
        Return si.nPos
    End Function

    Private Function GetScrollInfoY(ByVal hWnd As IntPtr) As SCROLLINFO
        Dim si As New SCROLLINFO
        si.fMask = SIF_ALL
        si.cbSize = Marshal.SizeOf(si)
        GetScrollInfo(hWnd, SBS_VERT, si)
        Return si
    End Function

    Friend Function GetScrollYmax(ByVal hWnd As IntPtr) As Int32
        Dim si As New SCROLLINFO
        si.fMask = SIF_ALL
        si.cbSize = Marshal.SizeOf(si)
        GetScrollInfo(hWnd, SBS_VERT, si)
        Return si.nMax
    End Function

    Friend Function GetScrollNpage(ByVal hWnd As IntPtr) As Int32
        Dim si As New SCROLLINFO
        si.fMask = SIF_ALL
        si.cbSize = Marshal.SizeOf(si)
        GetScrollInfo(hWnd, SBS_VERT, si)
        Return si.nPage
    End Function

    ' ============================================================================================================
    '  SET SCROLL POS
    ' ============================================================================================================
    Private Const WM_VSCROLL As Integer = &H115
    Private Const WM_HSCROLL As Integer = &H114
    Private Const WM_MOUSEWHEEL As Integer = &H20A
    Private Const WM_USER As Integer = &H400
    Private Const EM_SETSCROLLPOS As Integer = WM_USER + 222

    Private Declare Auto Function SendScrollPosMessage _
                                    Lib "user32.dll" Alias "SendMessage" _
                                    (ByVal hWnd As IntPtr, _
                                     ByVal Msg As Integer, _
                                     ByVal wParam As IntPtr, _
                                     ByRef lParam As Point) As Integer


    Friend Sub SetScrollPos(ByVal hwnd As IntPtr, ByVal x As Int32, ByVal y As Int32)
        SendScrollPosMessage(hwnd, _
                               EM_SETSCROLLPOS, _
                               New IntPtr(0), _
                               New Point(x, y))
    End Sub


    Friend Sub SetScrollPosY(ByVal hwnd As IntPtr, ByVal y As Int32)
        Dim x As Int32 = GetScrollPosX(hwnd)
        Dim si As SCROLLINFO = GetScrollInfoY(hwnd)
        If y < 65500 Then ' <--- This is a workaround because si.nMax is limited to 65535
            y = Math.Min(y, si.nMax - si.nPage)
        End If
        SetScrollPos(hwnd, x, y)
    End Sub


    ' ============================================================================================================
    '  IS SCROLLBAR VISIBLE
    ' ============================================================================================================
    Private Const GWL_STYLE As Integer = -16
    Private Const WS_HSCROLL As Integer = &H100000
    Private Const WS_VSCROLL As Integer = &H200000

    Private Declare Auto Function GetWindowLong _
                                   Lib "user32.dll" Alias "GetWindowLong" _
                                   (ByVal hWnd As IntPtr, _
                                    ByVal nIndex As Integer) As Integer

    Friend Function IsHorizontalScrollBarVisible(ByVal hwnd As IntPtr) As Boolean
        Dim dwStyle As Integer = GetWindowLong(hwnd, GWL_STYLE)
        Return (dwStyle And WS_HSCROLL) <> 0
    End Function

    Friend Function IsVerticalScrollBarVisible(ByVal hwnd As IntPtr) As Boolean
        Dim dwStyle As Integer = GetWindowLong(hwnd, GWL_STYLE)
        Return (dwStyle And WS_VSCROLL) <> 0
    End Function

End Module
